package schedulerTriage.bThreads;

import static bp.eventSets.EventSetConstants.*;
import schedulerTriage.events.Schedule;
import bp.BProgram;
import bp.BThread;
import bp.eventSets.RequestableEventSet;
import bp.exceptions.BPJException;

@SuppressWarnings("serial")
public class Process extends BThread {
	
	public int index;
	public int ticksInCycle;
	public int requestsPerCycle;
	
	public Process( int index, int ticksInCycle, int requestsPerCycle )
	{
		this.index = index;
		this.ticksInCycle = ticksInCycle;
		this.requestsPerCycle = requestsPerCycle;
	}
	
	int tickCounter;
	int remainingRequests;
	
	
	public void runBThread() throws BPJException 
	{	
		while ( true )
		{
			handleCycle();
		}
	}
	
	void handleCycle()
	{
		int requestsGrantedSoFar = 0;
		
		for ( int i = 0; i < ticksInCycle; ++i )
		{
			RequestableEventSet requests = new RequestableEventSet();
			RequestableEventSet blockedEvents;
			
			if ( requestsGrantedSoFar < requestsPerCycle )
			{
				requests.add( new Schedule( index, ticksInCycle - i ) );
				blockedEvents = new AllFurtherDeadlines( ticksInCycle - i );
			}
			else
			{
				blockedEvents = new RequestableEventSet();
			}

			BProgram.labelNextVerificationState( "( " + i + ", " + requestsGrantedSoFar + " )" );
			BProgram.bp.bSync( requests, all, blockedEvents );
			
			Schedule lastScheduling = (Schedule)BProgram.bp.lastEvent;
			
			if ( lastScheduling.requestedIndex == index )
			{
				++requestsGrantedSoFar;
			}
		}
		
		if ( requestsGrantedSoFar != requestsPerCycle )
		{
			BProgram.markNextVerificationStateAsBad( "Process #" + index + " not served!" );
			BProgram.bp.bSync( none, all, none );
		}
	}
}